using Toybox.System as Sys;
using Toybox.Time.Gregorian as Cal;

import Toybox.Lang;

//
//  In addition to providing simpleaccess to the time/date
//  fields we want in a format we want this class also
//  deals with Daylight Saving Time.
//
//  Define dst_info as the start (month, day of week, ordinal
//  of dow, and hour) and end of DST.  Comparing now() with
//  either of these markers when the ordinal is first, second, ,,.,
//  is easy'ish (find the dow for the first day of the month,
//  adjust it to match the dow, and then add (ordinal - 1)*seconds/week
//  to get the desired moment.
//
//  Then there are places like the EU which counts their ordinals
//  from the last dow of the month.  A little harder but all we do
//  is find the dow for the last day of the month (of course Feb.
//  is an issue, don't forget to account for leap years), adjust
//  it downward to match the dow, and then subtrace (ordinal-1)*sec/week
//  to get the desired moment.
//
//  (I know, nobody starts/ends DST in Feb. but let's get the algorithm
//  right just in case.)
//

class MyTime
{

var epoch;
var info;
var hour;
var hmins;
var abbrev;
var pm;

//
// [
//  mo - start month,
//  dw - start day of week,
//  n  - which dow (- is from end),
//  hr - start hour,
//  mo - end month,
//  dw - end day of week,
//  n  - which dow (- is from end),
//  hr - end hour
// ]
var dst_info as Array<Array<Number>> = [
//   mo dw   n hr   mo dw  ct hr
    [ 3, 1,  2, 2,  11, 1,  1, 2 ],         // USA/Canada, 2nd Sun Mar - 1st Sun Nov
    [ 3, 1, -1, 1,  10, 1, -1, 1 ]          // EU
];

var mo_days as Array<Number> = [  0,  31,  28,  31,  30,  31,  30,  31,  31,  30,  31,  30,  31 ];

public function initialize(epoch, tz_info as Array<Number>)
{

    //
    //  Following code is used to test the DST algorithm
    //
//    if (tz_info[0] != 0) {
//        epoch = to_utc(2024, 10, 27, 1, 0, 0);
//        C.pr_date("ref", epoch);
//    }

    me.epoch = epoch;
    var adj = dst_off(tz_info[3]);
    if (adj != null) {
        me.epoch = me.epoch.add(adj);
        abbrev = tz_info[2];
    } else {
        abbrev = tz_info[1];
    }
    me.info = Cal.info(me.epoch, Time.FORMAT_SHORT);
    me.hour = info.hour;

    if (me.hour >= 12) {
        me.hour -= 12;
        me.pm = true;
    } else {
        me.pm = false;
    }
    me.hmins = (me.hour * 60) + + info.min;
}

private function to_utc(yr, mon, day, hour, min, sec)
{

    var t = Cal.moment({ :year   => yr,
                         :month  => mon,
                         :day    => day,
                         :hour   => hour,
                         :minute => min,
                         :second => sec });
    t = t.subtract(new Time.Duration(Sys.getClockTime().timeZoneOffset));
    return t;
}

private function leap(yr)
{

    if ((yr % 400) == 0) {
        return 1; }
    if ((yr % 100) == 0) {
        return 0; }
    return ((yr % 4) == 0) ? 1 : 0;
}

//
//  Return moment representing the count'th dow of the
//    current month at hr hour
//
private function dom(mo, dow, count, hr)
{
    var delta;

    var t = to_utc(me.info.year, mo, 1, hr, 0, 0);
    var info = Cal.info(t, Time.FORMAT_SHORT);
    if (count < 0)
    {
        var days = mo_days[mo];
        if (mo == 2) {
            days += leap(info.year); }
        t = to_utc(me.info.year, mo, days, hr, 0, 0);
        info = Cal.info(t, Time.FORMAT_SHORT);
        delta = info.day_of_week - dow;
        if (delta < 0) {
            delta += 7; }
        t = t.subtract(new Time.Duration(delta * 24 * 60 * 60));
        t = t.subtract(new Time.Duration(((count + 1) * 7 * 24 * 60 * 60)));
    } else {
        delta = dow - info.day_of_week;
        if (delta < 0) {
            delta += 7; }
        t = t.add(new Time.Duration(delta * 24 * 60 * 60));
        t = t.add(new Time.Duration((count - 1) * 7 * 24 * 60 * 60));
    }
    return t;
}

private function before(mo, dow, count, hour)
{

    //
    // Slight optimization.  We only need to calculate moments
    // if the current month is the one we want
    //
    if (info.month < mo) {
        return true; }
    if (info.month != mo) {
        return false; }

    var d = dom(mo, dow, count, hour);
    return epoch.lessThan(d);
}

private function within(info as Array<Number>) as Boolean
{

    me.info = Cal.info(me.epoch, Time.FORMAT_SHORT);
    if (before(info[0], info[1], info[2], info[3])) {
        return false; }
    if (before(info[4], info[5], info[6], info[7])) {
        return true; }
    return false;
}

function dst_off(dst)
{

    if ((dst == -1) || (dst == -2)) {
        return null; }
    else if (dst == -3) {
        return new Time.Duration(60*60); }

    return within(dst_info[dst]) ? new Time.Duration(60*60) : null;
}

}
